import time
import remoteapi

"""
TestScenario.py

Unit test for Remote API that creates a single client and executes
several scenario-based PIData commands remotely.  It creates a
scenario, initializes values, checks values, then continues to
execute other scenario commands.
"""
# Make the wait times variables so that a very large numbers can be selected
# when running the script with valgrind

waitTimeShort = 1000
waitTimeLong = 10000

import RemoteApi_pb2
import remoteapi.PluginApi
def customAddData(piData, id, points):
	msg = RemoteApi_pb2.Command()
	msg.piData.platform.addData.id = id
	msg.piData.platform.addData.frame.system = 3 # LLA
	for inPoint in points:
		point = msg.piData.platform.addData.points.add();
		point.time = inPoint[0]
		point.position.x = inPoint[1]
		point.position.y = inPoint[2]
		point.position.z = inPoint[3]
	return remoteapi.PluginApi.ResponseHandler(piData.client_, piData.client_.send(msg),
		lambda r: r.response.returnCode)

# Adds data points to the given platform
def sendPlatformData(piData, platformId):
	points = []
	for time in range(0, 100 + 1):
		pct = time * 0.01
		# Store points in time, lat, lon, alt
		points.append([time, 1.0 * pct, 1.0 * pct, 1000.0 * pct])
	customAddData(piData, platformId, points)

# Checks the playing state of the clock
def clockIsPlaying(piData, state):
	f = piData.clockIsPlaying()
	if not f.isOk(waitTimeShort):
		raise Exception("Failed to check clockIsPlaying")
	if f.value() != state:
		raise Exception("Failed to get the correct state for clockIsPlaying")

# verify number of viewports
def verifyNumViewports(number):
	f = piCommon.getNumViewports();
	if not f.isOk(waitTimeShort):
		raise Exception("Failed to get the number of viewports")
	if f.value() != number:
		raise Exception("Did not get the correct number of viewports")

client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piCommon = pi.common()
piSimdis = pi.simdis()

pi.config().setResponsesEnabled(True)

f = piData.isScenarioInitialized()
if not f.isOk(waitTimeShort):
	raise Exception("Failed isScenarioInitialized")

if f.value():
	f = piData.deleteScenario()
	if not f.isOk(waitTimeLong):
		raise Exception("Failed to delete scenario")

f = piData.initializeScenario("Python Test", 0, 0, 0)
if not f.isOk(waitTimeLong):
	raise Exception("Failed to initialize scenario")

f = piData.setOriginLLA(0.1, 0.2, 1000)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set OriginLLA")

f = piData.getOriginLLA()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to receive Origin LLA")
if f.value()[0] != 0.1:
	raise Exception("Failed to receive expected latitude")
if f.value()[1] != 0.2:
	raise Exception("Failed to receive expected longitude")
if f.value()[2] != 1000:
	raise Exception("Failed to receive expected altitude")

f = piData.setReferenceYear(2013)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set ref year")
f = piData.getReferenceYear()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get ref year")
if f.value() != 2013:
	raise Exception("Failed to receive expected reference year")

f = piData.setDescription("This is a test description")
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set description")
f = piData.getDescription()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get description")
if f.value() != "This is a test description":
	raise Exception("Failed to receive expected description")

f = piData.setScenarioFilename("This is a test filename")
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set filename")
f = piData.getScenarioFilename()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get filename")
if f.value() != "This is a test filename":
	raise Exception("Failed to receive expected filename")

color = 0xffff80ff  # 0xRRGGBBAA
f = piData.setClassification('This is a test', color)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set classification")
f = piData.getClassification()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get classification")
if f.value()[0] != "This is a test":
	raise Exception("Failed to receive expected classification string")
if f.value()[1] != color:
	raise Exception("Failed to receive expected classification color")

f = piData.setWind(3.1, 0.21)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set wind")
f = piData.getWind()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get wind")
if f.value()[0] != 3.1:
	raise Exception("Failed to get expected wind speed")
if f.value()[1] != 0.21:
	raise Exception("Failed to get expected wind direction")

# Test Platforms
f = piData.createPlatform("Platform 1", "x", 47)
if not f.isOk(waitTimeLong):
	raise Exception("Failed to create platform")
platformId = f.value();

f = piData.updatePlatformHeader(platformId, "Platform 1", "x", 90)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to update platform header")

f = piData.getPlatformHeader(platformId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get platform header")
if f.value().originalId != 90:
	raise Exception("Failed to get the correct platform header")

f = piData.getObjectType(platformId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get platform object type")
if f.value() != 1:
	raise Exception("Failed to get the correct object type for platform")

f = piData.getObjectHost(platformId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get platform host id")
if f.value() != 0:
	raise Exception("Failed to get the correct host id for platform")

# Add some data points
sendPlatformData(piData, platformId)

# Test Magnetic Variance
f = piSimdis.getMagneticVarianceById(platformId, 1.0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get magnetic variance")

f = piSimdis.getMagneticVariance(0.1, 0.2, 1000, 1.0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get magnetic variance")

# Test RCS Pattern File
f = piData.setPlatformRCSPatternFileName(platformId, "$(SIMDIS_DIR)/demos/SIMDIS/import/ship_rcs_1.rcs")
if not f.isOk(waitTimeLong):
	raise Exception("Failed to set Platform RCS Pattern Filename")
if f.value() != 0:
	raise Exception("Failed to set Platform RCS Pattern Filename")

f = piData.getPlatformRCSPatternFileName(platformId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get Platform RCS Pattern Filename")
if f.value() != "$(SIMDIS_DIR)/demos/SIMDIS/import/ship_rcs_1.rcs":
	raise Exception("Failed to get Platform RCS Pattern Filename 2")

# Test Beams
f = piData.createBeam(platformId, "Beam 1", 1, 2.0, 3.0, 4)
if not f.isOk(waitTimeLong):
	raise Exception("Failed to create beam")
beamId = f.value();

f = piData.updateBeamHeader(beamId, "Beam 1", 1, 2.0, 3.0, 5)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to update beam header")

f = piData.getBeamHeader(beamId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get beam header")
if f.value().originalId != 5:
	raise Exception("Failed to get the correct beam header")

f = piData.getObjectType(beamId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get beam object type")
if f.value() != 2:
	raise Exception("Failed to get the correct object type for beam")

f = piData.getObjectHost(beamId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get beam host id")
if f.value() != platformId:
	raise Exception("Failed to get the correct host id for beam")

# Test Gates
f = piData.createGate(beamId, "Gate 1", 1, 6)
if not f.isOk(waitTimeLong):
	raise Exception("Failed to create gate")
gateId = f.value();

f = piData.updateGateHeader(gateId, "Gate 1", 1, 7)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to update gate header")

f = piData.getGateHeader(gateId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get gate header")
if f.value().originalId != 7:
	raise Exception("Failed to get the correct gate header")

f = piData.getObjectType(gateId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get gate object type")
if f.value() != 4:
	raise Exception("Failed to get the correct object type for gate")

f = piData.getObjectHost(gateId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get gate host id")
if f.value() != beamId:
	raise Exception("Failed to get the correct host id for gate")

# Test Target Beams
f = piData.createBeam(platformId, "Target Beam 1", 1, 2.0, 3.0, 444)
if not f.isOk(waitTimeLong):
	raise Exception("Failed to create target beam")
targetBeamId = f.value();

f = piData.setTarget(targetBeamId, platformId, 0.0);
if not f.isOk(waitTimeLong):
	raise Exception("Failed to set target at time 0.0")

f = piData.setTarget(targetBeamId, 0, 10.0);
if not f.isOk(waitTimeLong):
	raise Exception("Failed to set target at time 10.0")

f = piData.setTarget(targetBeamId, platformId, 20.0);
if not f.isOk(waitTimeLong):
	raise Exception("Failed to set target at time 20.0")

f = piData.getTargets(targetBeamId, -1.0, -1.0)
if not f.isOk(waitTimeLong):
	raise Exception("Failed to get target data")
if len(f.value()) != 3:
	raise Exception("getTargets: got the wrong length")
if f.value()[0].time != 0:
	raise Exception("getTargets: Entry 1 has the wrong time")
if f.value()[0].targetId != platformId:
	raise Exception("getTargets: Entry 1 has the wrong target ID")

# Test LOB
f = piData.createLob(platformId, "LOB 1", 0, 9)
if not f.isOk(waitTimeLong):
	raise Exception("Failed to create LOB")
lobId = f.value();

f = piData.updateLobHeader(lobId, "LOB 1", 1, 9)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to update LOB header")

f = piData.getLobHeader(lobId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get LOB header")
if f.value().originalId != 9:
	raise Exception("Failed to get the correct LOB header")

f = piData.getObjectType(lobId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get lob object type")
if f.value() != 64:
	raise Exception("Failed to get the correct object type for lob")

f = piData.getObjectHost(lobId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get lob host id")
if f.value() != platformId:
	raise Exception("Failed to get the correct host id for lob")

# Test lob draw style
f = piData.getLobDrawStyle(lobId, 0.0, 100.0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get lob draw styles")
# should be empty
if len(f.value()) != 0:
	raise Exception("Failed to get correct number of lob draw styles")
f = piData.addLobDrawStyle(lobId, 0.0, 1, 2, 3, 4, 5)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to add lob draw styles")
# should be empty
if f.value() != 0:
	raise Exception("Failed to add a lob draw style")
f = piData.getLobDrawStyle(lobId, 0.0, 100.0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get lob draw styles")
# should have one
if len(f.value()) != 1:
	raise Exception("Failed to get correct number of lob draw styles")

# Test laser
f = piData.createLaser(platformId, "Laser 1", 10, False, 0, [0,0,0], True)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to create laser")
laserId = f.value()
f = piData.getLaserHeader(laserId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get laser header")
if f.value().originalId != 10:
	raise Exception("Failed to get the correct laser header")
f = piData.updateLaserHeader(laserId, "NewLaserName")
if not f.isOk(waitTimeShort):
	raise Exception("Failed to update laser header")
f = piData.getLaserHeader(laserId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get laser header")
if f.value().name != "NewLaserName":
	raise Exception("Failed to get the correct laser header")
f = piData.getObjectType(laserId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get the laser object type")
if f.value() != 32:
	raise Exception("Failed to get the correct object type for laser")
f = piData.getObjectHost(laserId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get the laser host id")
if f.value() != platformId:
	raise Exception("Failed to get the correct host id for laser")

# Test getting laser data
f = piData.getLaserData(laserId, 0.0, 100.0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get laser data")
# Should have no data at this point
if len(f.value()) != 0:
	raise Exception("Failed to get empty laser data")
f = piData.getLaserStyles(laserId, 0.0, 100.0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get laser styles 1")
# Should have no style at this point
if len(f.value()) != 0:
	raise Exception("Failed to get empty laser styles")
# Add some data
f = piData.addLaserData(laserId, 10.0, 0,0,0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to add laser data")
f = piData.addLaserStyle(laserId, 10.0, 100, 0,0,0, 10)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to add laser style")
# Check for data again now that some has been added
f = piData.getLaserData(laserId, 0.0, 100.0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get laser data")
# Should have no data at this point
if len(f.value()) != 1:
	raise Exception("Failed to get correct amount of laser data")
f = piData.getLaserStyles(laserId, 0.0, 100.0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get laser styles 2")
# Should have no style at this point
if len(f.value()) != 1:
	raise Exception("Failed to get correct amount of laser styles")

# Test getObjectList
f = piData.getObjectList()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get object list")
if len(list(f.value())) != 6:
	raise Exception("Failed to get correct number of objects")
if not platformId in f.value():
	raise Exception("Failed to find the platform in the list")
if not beamId in f.value():
	raise Exception("Failed to find the beam in the list")
if not gateId in f.value():
	raise Exception("Failed to find the gate in the list")
if not lobId in f.value():
	raise Exception("Failed to find the lob in the list")
if not laserId in f.value():
	raise Exception("Failed to find the laser in the list")
if not targetBeamId in f.value():
	raise Exception("Failed to find the target beam in the list")

# Test findObject
f = piData.findObject(1, "Platform 1")
if not f.isOk(waitTimeLong):
	raise Exception("Failed to find object")
if len(f.value()) != 1:
	raise Exception("Failed to get the correct number of objects")
if not platformId in f.value():
	raise Exception("Failed to find the platform in the list")
# Test findObject where it should find nothing
f = piData.findObject(1, "Not an object name")
if not f.isOk(waitTimeLong):
	raise Exception("Failed to find object")
if len(f.value()) != 0:
	raise Exception("Failed to get an empty list")

# test flushEntityData
f = piData.flushEntityData(platformId, 0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to flush entity data")
if f.value() != 0:
	raise Exception("Failed to flush entity data")
# put the platform data back in
sendPlatformData(piData, platformId)

# test set history lengths
f = piData.setHistoryLengths(platformId, 5, 6)
if not f.isOk(waitTimeLong):
	raise Exception("Failed to set history lengths")
if f.value() != 0:
	raise Exception("Failed to set history lengths")
f = piData.getHistoryLengths(platformId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get history lengths")
if f.value().points != 5:
	raise Exception("Failed to get the correct history points length")
if f.value().seconds != 6:
	raise Exception("Failed to get the correct history seconds length")

# test getGenericDataList
f = piData.getGenericDataList()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get generic data list")
# should be empty
if len(list(f.value())) != 0:
	raise Exception("Failed to get correct generic data list size")
# test getCurrentGenericData
f = piData.getCurrentGenericData(platformId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get the current generic data")
# should be empty
if len(f.value()) != 0:
	raise Exception("Failed to get correct generic data list size")
# add a generic data point
f = piData.addGenericData(platformId, 0.0, "Agent", "47" )
if not f.isOk(waitTimeShort):
	raise Exception("Failed to add generic data")
if f.value() != 0:
	raise Exception("Failed to add generic data")
# List should now have one
f = piData.getGenericDataList()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get generic data list")
# should be one
if len(list(f.value())) != 1:
	raise Exception("Failed to get correct generic data list size")
# current generic data should now have one
f = piData.getCurrentGenericData(platformId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get the current generic data")
# should be one
if len(f.value()) != 1:
	raise Exception("Failed to get correct generic data list size")

# test deleteGenericData
f = piData.deleteGenericData(platformId, "Agent")
if not f.isOk(waitTimeShort):
	raise Exception("Failed to delete generic data")
if f.value() != 0:
	raise Exception("Failed to delete generic data")

# test retrieveCurrentCategoryData
f = piData.retrieveCurrentCategoryData(platformId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to retrieve current category data")
# should be empty
if len(f.value()) != 0:
	raise Exception("Failed to get correct current category data list size")
# add a category data point
f = piData.addCategoryData(platformId, 0.0, "Agent", "47" )
if not f.isOk(waitTimeShort):
	raise Exception("Failed to add category data")
if f.value() != 0:
	raise Exception("Failed to add category data")
# List should now have one
f = piData.retrieveCurrentCategoryData(platformId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to retrieve current category data")
# should be one
if len(f.value()) != 1:
	raise Exception("Failed to get correct current category data list size")

# test getEntitiesByCategoryString
f = piData.getEntitiesByCategoryString("Agent(1)~47(1)")
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get entities by category string")
# should not be empty
if len(f.value()) != 1:
	raise Exception("Failed to get correct number of entities by category string")
if not platformId in f.value():
	raise Exception("Failed to find the platform in the list by category string")

# test getEntitiesByCategoryAndValue
f = piData.getEntitiesByCategoryAndValue("Agent", "47",  1)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get entities by category and value")
# should not be empty
if len(f.value()) != 1:
	raise Exception("Failed to get correct number of entities by category and value")
if not platformId in f.value():
	raise Exception("Failed to find the platform in the list by category and value")

# Should be in file mode; finish loading and test the clock
f = piData.clockAutoConfigureBeginEnd()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to clockAutoConfigureBeginEnd")
# should be empty
if f.value() != 0:
	raise Exception("Failed to clockAutoConfigureBeginEnd")

f = piData.clockGetBegin()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to clockGetBegin")
# should be zero seconds
if f.value() != 0.0:
	raise Exception("Failed to get expected clockGetBegin time")

# check the start and end times
f = piData.clockGetEnd()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to clockGetEnd")
# should be 100 seconds
if f.value() != 100.0:
	raise Exception("Failed to get expected clockGetEnd time")

# double the step size
f = piData.clockGetStep()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to clockGetStep")
stepSize = f.value()
f = piData.clockSetStep(2.0 * stepSize)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to clockSetStep")
f = piData.clockGetStep()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to clockGetStep")
if f.value() != 2.0 * stepSize:
	raise Exception("Step size not the expected value")

# Clock should not be playing
clockIsPlaying(piData, False)

# Start the Clock
f = piData.clockPlay()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to clockPlay")
# should be no errors
if f.value() != 0:
	raise Exception("Failed to clockPlay")

# Clock should be playing
clockIsPlaying(piData, True)

# Stop the Clock
f = piData.clockStop()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to clockStop")
# should be no errors
if f.value() != 0:
	raise Exception("Failed to clockStop")

# Clock should not be playing
clockIsPlaying(piData, False)

# Verify action state
f = piCommon.actionState("Quit")
if not f.isOk(waitTimeShort):
	raise Exception("Failed to actionState")
if f.value().exist != True:
	raise Exception("The Quit action should exist")

f = piCommon.actionState("ActionThatShouldNotExist")
if not f.isOk(waitTimeShort):
	raise Exception("Failed to actionState")
if f.value().exist != False:
	raise Exception("The action ActionThatShouldNotExist somehow exists")

# Verify Hot Keys
f = piCommon.saveHotKeyFile("$(SIMDIS_DIR)/OkToDeleteThisFile.qhotkey")
if not f.isOk(waitTimeLong):
	raise Exception("Failed to save hot key file")

f = piCommon.loadHotKeyFile("$(SIMDIS_DIR)/OkToDeleteThisFile.qhotkey")
if not f.isOk(waitTimeLong):
	raise Exception("Failed to load hot key file")

f = piCommon.loadHotKeyFile("$(SIMDIS_DIR)/ThisShouldFaildToLoad.qhotkey")
if not f.waitForResponse(waitTimeLong):
	raise Exception("Failed to load hot key file")
if f.value() == 0:
	raise Exception("Somehow loaded a file that does not exist")

f = piCommon.getHotKey("Quit", 0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get hot key")
hotKey = f.value();

f = piCommon.setHotKey("Quit", hotKey, 0, True)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set hot key")
hotKey = f.value();

# Verify Preference files
f = piCommon.savePrefsFile("$(SIMDIS_DIR)/OkToDeleteThisFile.prefs")
if not f.isOk(waitTimeLong):
	raise Exception("Failed to save preference file")

f = piCommon.loadPrefsFile("$(SIMDIS_DIR)/OkToDeleteThisFile.prefs")
if not f.isOk(waitTimeLong):
	raise Exception("Failed to load preference file")

# verify Overlay text
f = piCommon.createOverlayText("Test", 50, 50)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to create overlay text")
overlayId = f.value();

f = piCommon.updateOverlayText(overlayId, "Test 2", 25, 25)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to update overlay text")

f = piCommon.getOverlayText(overlayId)
if not f.waitForResponse(waitTimeLong):
	raise Exception("Failed to get overlay text")
if f.value().text != "Test 2":
	raise Exception("Failed to get the correct text")

f = piCommon.removeOverlayText(overlayId)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to remove overlay text")

# verify viewport routines

# verify initial state
verifyNumViewports(1)

# make the first viewport
f = piCommon.createViewport(1, "1", 0.0, 50.0, 0.0, 50.0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to create viewport")
verifyNumViewports(2)
f = piCommon.getViewport(1)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get viewport 1")
if f.value() != "1":
	raise Exception("Failed to get the expected viewport name")

# make a second view port
f = piCommon.createViewport(1, "2", 50.0, 50.0, 0.0, 50.0)
if not f.waitForResponse(waitTimeShort):
	raise Exception("Failed to get overlay text")
verifyNumViewports(3)
f = piCommon.getViewport(2)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to get viewport 2")
if f.value() != "2":
	raise Exception("Failed to get the expected viewport name")

# Set and toggle view port
f = piCommon.setViewport("1")
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set viewport 1")

f = piCommon.toggleViewport("1", 0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to toggle viewport 1")

time.sleep(1)

f = piCommon.toggleViewport("1", 1)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to toggle viewport 1")

f = piCommon.nextViewport()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to go to next viewport")

# delete viewport
f = piCommon.deleteViewport("1")
if not f.isOk(waitTimeShort):
	raise Exception("Failed to delete viewport 1")
verifyNumViewports(2)

# delete all remaining view ports
f = piCommon.deleteAllViewports()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to delete all viewports")
verifyNumViewports(1)

# verify checkEarthEclipse
f = piCommon.checkEarthEclipse(platformId)
if not f.waitForResponse(waitTimeShort):
	raise Exception("Failed to check Earth eclipse")
if f.value() == -1:
	raise Exception("Failed to check Earth eclipse")

# verify convertTime()
f = piData.convertTime(2019, 1.0, 2018)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to convertTime()")
if f.value() != (86400.0 * 365 + 1.0):
	raise Exception("convertTime() returned unexpected value")

# Set up convertToScenarioTime()
f = piData.setReferenceYear(2013)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set ref year")
f = piData.convertToScenarioTime(2014, 1.0)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to convertToScenarioTime()")
if f.value() != (86400.0 * 365 + 1.0):
	raise Exception("convertToScenarioTime() returned unexpected value")

client.close()
print("Test successful")
