import zmq
import socket
import Connection_pb2
import RemoteApi_pb2
from . import CommandReplyClient
import remoteapi.PluginApi

# The health of the connection and of the last message sent/received
CONNECTION_OK = 0  # The requested action completed with no errors
CONNECTION_BLOCKED = 1  # The requested action would have blocked
CONNECTION_INVALID = 2  # Connection went permanently bad
CONNECTION_MISSING_MESSAGE_HEADER = 3  # The message header is missing or truncated
CONNECTION_RESPONSE_ALREADY_PASSED = 4 # The requested response has already passed

def stateToString(state):
	if state is CONNECTION_OK:
		return "OK"
	elif state is CONNECTION_BLOCKED:
		return "BLOCKED"
	elif state is CONNECTION_INVALID:
		return "INVALID"
	elif state is CONNECTION_MISSING_MESSAGE_HEADER:
		return "MISSING_MESSAGE_HEADER"
	elif state is CONNECTION_RESPONSE_ALREADY_PASSED:
		return "RESPONSE_ALREADY_PASSED"
	return str(state)

class Client(object):
	address = "localhost"
	commandPort = 0
	responsePort = 0
	discoveryPort = 0
	clientId = None
	serverId = None
	crc_ = None

	def __init__(self, zmqContext, address, commandPort, responsePort):
		self.crc_ = CommandReplyClient.CommandReplyClient(zmqContext, address, commandPort, responsePort)
		pass

	def __del__(self):
		self.close(500)

	def isValid(self):
		"""Returns true if the connection is currently open and valid"""
		return self.crc_ is not None

	def close(self, timeoutMs=500):
		"""Gracefully close out the connection, telling SIMDIS to close out too"""
		self.crc_ = None
		if self.clientId is None or self.serverId is None:
			return
		dc = DiscoveryClient()
		dc.discoveryPort = self.discoveryPort
		dc.discoveryAddress = self.address
		dc.closeClient(self.serverId, self.clientId)
		self.clientId = self.serverId = None

	def send(self, remoteApiCommand):
		if not self.isValid():
			return (CONNECTION_INVALID, None)
		commandString = remoteApiCommand.SerializeToString()
		rvAndSeq = self.crc_.sendMessage(commandString, 0)
		return (rvAndSeq[0], rvAndSeq[1])

	def getResponse(self, sequenceNumber, waitMilliseconds=5000):
		if not self.isValid():
			return (CONNECTION_INVALID, None)
		rv = self.crc_.getResponse(sequenceNumber, waitMilliseconds)
		if len(rv) != 2 or rv[0] != CONNECTION_OK or rv[1] is None:
			return (rv[0], None)
		msg = RemoteApi_pb2.Response()
		msg.ParseFromString(rv[1])
		return (CONNECTION_OK, msg)

	def flushResponse(self, sequenceNumber):
		if not self.isValid():
			return CONNECTION_INVALID
		return self.crc_.flushResponse(sequenceNumber)

	def flushAllResponses(self, waitMilliseconds=0):
		if not self.isValid():
			return CONNECTION_INVALID
		return self.crc_.flushAllResponses(waitMilliseconds)

	def pluginApi(self):
		return PluginApi.PluginApi(self)

class DiscoveryClient(object):
	"""Communicates with the SIMDIS Remote API Discovery Server using ZeroMQ
	
	Attributes:
		description	Custom description of your client
		hostname	Hostname of your client
		ipAddress	IP Address of your client (IPv4 expected)
		timeout	Milliseconds to wait for responses
		protocol	ZeroMQ protocol to connect; typically "tcp"
		lastError	Last error encountered, in string format
		context	ZeroMQ context
		discoveryAddress	Address of discovery server
		discoveryPort	Port the discovery server is listening on
	"""
	description = "Python Client"
	hostname = socket.gethostname()
	ipAddress = socket.gethostbyname(hostname)
	timeout = 5000
	protocol = "tcp"
	lastError = ""
	context = zmq.Context()
	discoveryAddress = "localhost"
	discoveryPort = 8482

	def createClient(self):
		"""Create a new connection to a running SIMDIS Remote API Server.
		Args:
			address (str): Address of the SIMDIS server
			port (int): Port on which SIMDIS is listening with the discovery service
		Returns: None on error, else ....
		"""
		msg = Connection_pb2.Command()
		msg.open.clientHostname = self.hostname
		msg.open.clientIpAddress = self.ipAddress
		msg.open.clientDescription = self.description
		response = self.sendRequest_(msg, Connection_pb2.Response())
		if response is None:
			self.lastError = "Did not receive response from discovery server"
			return None

		client = Client(self.context, "{0}://{1}".format(self.protocol, self.discoveryAddress),
			response.ports.commandPort, response.ports.responsePort)
		client.discoveryPort = self.discoveryPort
		client.clientId = response.ports.clientId
		client.serverId = response.ports.serverId

		return client

	def closeClient(self, serverId, clientId):
		"""Closes a previously opened connection to a running SIMDIS Remote API Server.
		Args:
			serverId (int): Server ID of SIMDIS host
			clientId (int): Assigned client ID from SIMDIS on createConnection
		"""
		msg = Connection_pb2.Command()
		msg.close.closeClientId = clientId
		msg.close.closeServerId = serverId
		self.sendRequest_(msg, Connection_pb2.Response())

	def poll_(self, socket, timeout):
		poller = zmq.Poller()
		poller.register(socket, zmq.POLLIN)
		socks = dict(poller.poll(timeout))
		if not socket in socks:
			return None
		return socket.recv_multipart()

	def sendRequest_(self, command, response):
		self.lastError = ""

		socket = self.context.socket(zmq.REQ)
		socket.setsockopt(zmq.LINGER, 0)
		socket.setsockopt(zmq.RCVHWM, 10000)
		socket.setsockopt(zmq.RCVBUF, 500000)
		socket.connect("{0}://{1}:{2}".format(self.protocol, self.discoveryAddress, self.discoveryPort))
		strValue = command.SerializeToString()
		socket.send(strValue)
		responseStr = self.poll_(socket, self.timeout)
		socket.close()
		if responseStr is None or len(responseStr) == 0 or response is None:
			return None
		response.ParseFromString(responseStr[0])
		return response
